# Fachschaftsminecraftserver

Maintainer: Tobias Müller <tmueller@cl.uni-heidelberg.de>

Dieses Image basiert auf der Arbeit von https://github.com/Phyremaster/papermc-docker und wurde so modifiziert, dass nur noch ein Befehl genutzt werden muss, um das Image auf einem beliebigen Server zu starten.

## :whale: Setup mit Docker

Die Anwendung ist dockerisiert. Images werden manuell vom Maintainer gebaut und in die Container Registry hochgeladen.

Wir empfehlen, `docker-compose` zu verwenden, um die Anwendung zu deployen, zum Beispiel mit der folgenden `docker-compose.yml`:

```yaml
version: "3"
services:
  papermc:
    image: gitlab.cl.uni-heidelberg.de:5050/tmueller/clspigot-docker:latest
    ports:
      - 25565:25565 # <Host-Port>:<Container-Port>, letzterer ist immer 25565
    volumes:
      - /hosts/location/of/papermc-files:/papermc
      - /hosts/location/of/backups:/Backup
    restart: on-failure
```

Um den Container zum Laufen zu bringen, muss man sich an dem Server mit dem folgenden Befehl erst mal an der Docker-Registry anmelden:

```
# docker login gitlab.cl.uni-heidelberg.de:5050
```

Das hier verwendete Passwort (bzw. Authorization Token) wird irgendwo im Klartext gespeichert, wenn man das Image also auf einen geteilten Server deployt, sollte man dafür einen geteilten Account verwenden (und nicht seinen persönlichen).

### Anmerkungen

* Bitte den Server nur über die Konsole in Minecraft stoppen, um Fehler insbesondere an der Welt zu vermeiden.
* Sofern die DynMap über den internen Webserver des Plugins ausgeliefert werden soll, muss auch noch der Container-Port 8123 nach außen gemappt werden. Der Maintainer rät allerdings dazu, den aktuellen Nginx-Container stattdessen zu nutzen (und DynMap entsprechend anzupassen). Dabei können selbst bei rund 10 Aufrufen pro Minute erheblich ressourcen gespart werden – ein Java-basierter Webserver ist keine gute Idee.

<!-- TODO

### Environment Variablen
Um eine Env Variable festzulegen gilt das folgenden Muster: `-e <NAME>="<VALUE>"` where `<NAME>` 
Der Platzhalter `<Name>`Wird durch den Namen der Env Variable ersetzt und `<value>` wird mit dem zuzuweisenden Wert ersetzt.
- Minecraft Version
  - **Name:** `MC_VERSION`
  - Legt die zu verwendene MC-Version fest.
  - !Wichtig! Die spezifizierte Version muss auch einen PaperMC Release haben, da sonst der Build fehlschlägt
  - [Aktuelle PaperMC-Builds](https://papermc.io/downloads) (Zuletzt 1.16.4) 
  - `-e MC_VERSION="<latest>"`
  - Die Buildnummern (ie. #355) sind normalerweise irrelevant hier kann man es bei latest belassen, falls jedoch ein spezifischer Build verwendet werden soll, lässt sich das durch den Parameter erreichen:
  - `-e PAPER_BUILD="<latest>"`
- RAM
  - **Name:** `MC_RAM`
  - Legt die Speichermenge für java fest. Zahl muss M für Megabytes oder G für Gigabytes enthalten.
  - Für die Flags der Java VM die ich aktuell nutze empfielt es sich 6GB minimum zuzuweisen oder maximal 10GB
  - `-e MC_RAM="<4G>"`
- Java options
  - **Name:** `JAVA_OPTS`
  - Javaspezifische Parameter, nur ändern wenn es notwendig ist und man weis was man tut!
  - Ausnahme, wenn der Server mit weniger RAM laufen soll eignet es sich hier die für spigot üblichen flags (siehe SpigotMC Docs) einzusetzen um aikars flags zu überschreiben
  - Mit genug Erfahrung können auch die Werte von den flags angepasst werden, aber auch hier gilt: Vorsicht nur anpassen wenn man weis was man da tut
  - `-e JAVA_OPTS="-XX:+UseG1GC -XX:+ParallelRefProcEnabled -XX:MaxGCPauseMillis=200 -XX:+UnlockExperimentalVMOptions -XX:+DisableExplicitGC -XX:+AlwaysPreTouch -XX:G1NewSizePercent=30 -XX:G1MaxNewSizePercent=40 -XX:G1HeapRegionSize=8M -XX:G1ReservePercent=20 -XX:G1HeapWastePercent=5 -XX:G1MixedGCCountTarget=4 -XX:InitiatingHeapOccupancyPercent=15 -XX:G1MixedGCLiveThresholdPercent=90 -XX:G1RSetUpdatingPauseTimePercent=5 -XX:SurvivorRatio=32 -XX:+PerfDisableSharedMem -XX:MaxTenuringThreshold=1 -Dusing.aikars.flags=https://mcflags.emc.gs -Daikars.new.flags=true"`

# Befehle
Innerhalb von Minecraft kann per `/backup` der Server gesichert werden.
Zum stoppen des Servers einfach über docker den container stoppen.

Zur Konsole vom Minecraftserver kommt man indem man diesen befehl eingibt: `docker exec -it <container name> /bin/bash`
und danach screen -r mc

# Technical
This project *does **NOT** redistribute the Minecraft server files*. Instead, the (very small) script that is inside of the image, `papermc.sh`, downloads these files from their official sources during installation.

**PLEASE NOTE:** This is an unofficial project. I did not create PaperMC. [This is the official PaperMC website.](https://papermc.io/)
-->

## :books: Weitere Ressourcen 

Projektsites des ursprünglichen Images von Phyremaster:

- [GitHub](https://github.com/Phyremaster/papermc-docker)
- [Docker Hub](https://hub.docker.com/r/phyremaster/papermc)
